local version = 2
--local debug_ = 1 -- hardcoded captchaID 11111111 with answer '2C888V'

local errors = {
	  "0001 API key doesn't exist"
	, "0002 API key not found"
	, "0003 Active API key not found"
	, "0004 API key deactivated by owner"
	, "0005 No user found"
	, "0006 No data found"
	, "0007 No ID found"
	, "0008 No captcha found"
	, "0009 No image found"
	, "0010 Image size not allowed"
	, "0011 Balance insufficient"
	, "0012 Already done."
	, "0013 No answer found."
	, "0014 Captcha already answered."
	, "0015 Captcha submitted too quickly."
	, "0016 JD Check active."
	, "0017 Unknown problem."
	, "0018 No ID found."
	, "0019 Incorrect answer."
	, "0020 Not filed on time (wrong UserID)"
	, "0021 Link not allowed."
	, "0022 Submit denied."
	, "0023 Solve denied."
	, "0024 Not enough credits."
	, "0025 No input found."
	, "0026 No conditions accepted."
	, "0027 No couponcode in the database found."
	, "0028 Already used coupon code."
	, "0029 Maxtimeout under 60 seconds."
	, "0030 User not found."
	, "0031 An account is not yet 24 hours in system."
	, "0032 An account does not have the full rights."
	, "0033 Plugin needed a update."
	, "0034 No HTTPS allowed."
	, "0035 No HTTP allowed."
	, "0036 Source not allowed."
	, "0037 Transfer denied."
}
errors['0002 Kein API Key gefunden.'] = errors[2]
errors['0003 Keinen aktiven API Key gefunden.'] = errors[3]
errors['0022 Einreichen verboten.'] = errors[22]

return function (cfile, conf)
	usd.savetolog('9kw_eu.lua v'..version)
	local acc = Accounts['9kw.eu']
	if not acc then return usd.savetolog("Accounts['9kw.eu'] not found") end
	local imageTypes = {jpeg=true, png=true, gif=true, bmp=true}
	local cfileType
	cfile, cfileType = plg.setproperext(cfile)
	if not cfile or not imageTypes[cfileType] then
		error('no captcha or wrong type: '..tostring(cfileType))
	end

	local logName = '9kw_eu_v'..version..'.html'
	local oldRef, oldCookies = usd.getlocation(), usd.getcookie()
	usd.setreferer(''); usd.setcookie('')

	local url9kw = "http://www.9kw.eu/index.cgi"
	local apikey = acc[1].login
	local source = 'usdLua'
	local captchaID
	local servercheck = usd.get('http://www.9kw.eu/grafik/servercheck.txt')
	debugLog(servercheck)
	local serverdata = {servercheck=servercheck}
	for k,v in servercheck:gmatch('(%w+)=(%d+)') do serverdata[k]=tonumber(v) end

	local function makeRequest(data, post)
		if post then
			local tmp = tableCopy(data)
			tmp['file-upload-01'] = tmp['file-upload-01'] and '<skipped>' or '<none>'
			debugLog(serialize(tmp))
		end
		return tableConcat(tableCombine(data, {apikey=apikey,source=source,debug=debug_}), '=',post and '\n' or '&',not post and escapeUC)
	end
	local function api(data, post)
		data = makeRequest(data, post)
		local page = post and usd.post(url9kw, data) or usd.get(url9kw..'?'..data)
		debugSaveInc(logName, page, captchaID or serialize(serverdata,nil,true))
		local err, msg = page:match('^(%d%d%d%d) ([^\r\n]+)$')
		if err then
			err = tonumber(err)
			if errors[page] then
				usd.savetolog('Error: '..errors[page])
				return page, errors[page]
			end
			if errors[err] then
				debugLog(page)
				usd.savetolog('Possible error: '..errors[err])
			end
		end
		return page, false
	end

	local balance, err = api {action='usercaptchaguthaben'}
	balance = tonumber(balance)
	debugLog('balance = '..tostring(balance))
	if not balance then return debugLog("Error: can't check balance") end
	if balance < 10 then return debugLog('Error: balance < 10') end

	--api {action='userservercheck'}

	usd.setstatus('sending captcha to 9kw.eu...')
	local expires = conf.expires and minMax(60,3999, conf.expires) or nil
	if expires then debugLog('expires='..expires) end
	local timeout = minMax(60,3999, expires or tonumber(conf.timeout) or 300)
	debugLog('timeout='..timeout)

	local continue = true
	if conf.evaluateTimeout then
		if type(conf.evaluateTimeout)=='function' then
			continue = conf.evaluateTimeout(timeout, serverdata)
		else
			continue = serverdata.queue / (serverdata.worker+1) < serverdata.avg15m
		end
	end

	if not continue then
		debugLog('Skip: not enough workers?')
		return ''
	end

	captchaID = api({action='usercaptchaupload', ['file-upload-01']=base64.encode(fileRead(cfile)), base64=1, maxtimeout=timeout, min_len=conf.min, max_len=conf.max}, true)
	local tmp = URI.parse(usd.getlocation())
	--if tmp.q.action=='userhistorydetail' then captchaID = tmp.q.id end
	captchaID = tonumber(captchaID)
	if not captchaID then return usd.savetolog('no captchaID!')	end
	if captchaID==11111111 then usd.savetolog('debug captchaID!')	end

	local timeStart = os.time()

	usd.setstatus('waiting captcha answer...')
	usd.waittime(10)
	local answer
	local tmp = {}
	--for i=1, math.floor((timeout-10)/5)+1 do
	for i=1, 66 do
		local data = makeRequest {action='usercaptchacorrectdata', id=captchaID, info=1}
		answer = usd.get(url9kw..'?'..data)
		table.insert(tmp, usd.getheaders()..answer)
		if answer ~= 'NO DATA' then break end
		usd.waittime(5)
	end

	local seconds = os.time()-timeStart
	local logFile = usd.workdir..[[Lua\9kw_eu.csv]]
	local headers = {'date','time','seconds','worker','workermouse','workerconfirm','workertext','queue','queue1','queue2','inwork','avg5m','avg15m','avg1h','avg24h'}
	if conf.log then
		local tmp, tmp2 = {}, {os.date('%Y-%m-%d;%H:%M:%S'),seconds}
		if (fileSize(logFile) or 0) < 100 then
			table.insert(tmp,'sep=;')
			table.insert(tmp,'"'..table.concat(headers,'";"')..'"')
		end
		for k,v in ipairs(headers) do if serverdata[v] then table.insert(tmp2, serverdata[v]) end end
		table.insert(tmp, table.concat(tmp2,';'))
		fileAppend(logFile, table.concat(tmp,'\r\n'),'\r\n')
	end
	debugSaveInc(logName, table.concat(tmp, '\r\n'..('-'):rep(88)..'\r\n'), captchaID)

	if expires and os.time()-timeStart > expires then debugLog('captcha expired!') end

	local function makeFB(correct, captchaID, msg)
		local url = url9kw..'?'..makeRequest{action='usercaptchacorrectback',correct=correct, id=captchaID}
		return function()
			local oldRef, oldCookies = usd.getlocation(), usd.getcookie()
			usd.setreferer(''); usd.setcookie('')
			debugSaveInc(logName, usd.get(url), (msg and msg..' ' or '')..'captchaID = '..captchaID..' correct = '..correct)
			usd.setreferer(oldRef); usd.setcookie(oldCookies)
		end
	end

	usd.setreferer(oldRef)
	usd.setcookie(oldCookies)
	usd.setstatus('')

	local badAnswers = {'NO DATA', 'ERROR NO USER'}

	answer = answer or ''
	return not tableContains(badAnswers, answer) and answer, {ok=makeFB(1, captchaID, 'ok'), complain=makeFB(2, captchaID, 'complain')}
end
